﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.WpfDesktopEdition;
using ThinkGeo.MapSuite.Core;
using System.Drawing.Printing;
using System.Collections.ObjectModel;

namespace PrintWmsDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        PrinterInteractiveOverlay printerOverlay;
        PagePrinterLayer pagePrinterLayer;
        MapPrinterLayer mapPrinterLayer;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void WindowLoaded(object sender, RoutedEventArgs e)
        {
            map.MapUnit = GeographyUnit.Meter;
            map.ZoomLevelSet = new PrinterZoomLevelSet(map.MapUnit, PrinterHelper.GetPointsPerGeographyUnit(map.MapUnit));
            map.MinimumScale = map.ZoomLevelSet.ZoomLevel20.Scale;
            map.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.StandardColors.LightGray);

            printerOverlay = new PrinterInteractiveOverlay();

            pagePrinterLayer = new PagePrinterLayer(PrinterPageSize.AnsiA, PrinterOrientation.Portrait);
            printerOverlay.PrinterLayers.Add(pagePrinterLayer);

            mapPrinterLayer = new MapPrinterLayer();
            mapPrinterLayer.MapUnit = GeographyUnit.Meter;
            mapPrinterLayer.BackgroundMask = new AreaStyle(new GeoPen(GeoColor.StandardColors.Black, 1));
            mapPrinterLayer.MapExtent = new RectangleShape(-13086298.60, 7339062.72, -8111177.75, 2853137.62);
            pagePrinterLayer.Open();
            var pageBoundingbox = pagePrinterLayer.GetPosition(PrintingUnit.Inch);
            mapPrinterLayer.SetPosition(8, 7, pageBoundingbox.GetCenterPoint().X, pageBoundingbox.GetCenterPoint().Y + 1, PrintingUnit.Inch);
            WmsRasterLayer wmsImageLayer = CreateWmsRasterLayer();
            mapPrinterLayer.Layers.Insert(0, wmsImageLayer);

            printerOverlay.PrinterLayers.Add("MapLayer", mapPrinterLayer);
            map.InteractiveOverlays.Add(printerOverlay);
            map.CurrentExtent = AreaBaseShape.ScaleUp(pagePrinterLayer.GetPosition(), 10).GetBoundingBox();
        }

        private void PrintMap(object sender, RoutedEventArgs e)
        {
            var printDocument = new PrintDocument
            {
                DefaultPageSettings =
                {
                    Landscape =true,
                    PaperSize = new PaperSize("AnsiA", 768, 1024)
                }
            };

            var printerGeoCanvas = new PrinterGeoCanvas { DrawingArea = printDocument.DefaultPageSettings.Bounds, Dpi = 100 };
            printerGeoCanvas.BeginDrawing(printDocument, pagePrinterLayer.GetBoundingBox(), map.MapUnit);
            var labelsInAllLayers = new Collection<SimpleCandidate>();
            mapPrinterLayer.Draw(printerGeoCanvas, labelsInAllLayers);
            printerGeoCanvas.Flush();

            var printPreviewDialog = new System.Windows.Forms.PrintPreviewDialog { Document = printDocument };
            printPreviewDialog.ShowDialog();
        }

        private WmsRasterLayer CreateWmsRasterLayer()
        {
            WmsRasterLayer wmsImageLayer = new WmsRasterLayer(new Uri("http://howdoiwms.thinkgeo.com/WmsServer.aspx"));
            wmsImageLayer.UpperThreshold = double.MaxValue;
            wmsImageLayer.LowerThreshold = 0;
            wmsImageLayer.Crs = "EPSG:900913";

            wmsImageLayer.Open();
            foreach (string layerName in wmsImageLayer.GetServerLayerNames())
            {
                wmsImageLayer.ActiveLayerNames.Add(layerName);
                wmsImageLayer.ActiveStyleNames.Add("Simple");
            }

            return wmsImageLayer;
        }
    }
}
